### A Pluto.jl notebook ###
# v0.15.1

using Markdown
using InteractiveUtils

# ╔═╡ f2385a7e-64f1-47d4-bf9c-59855be81eca
begin
	import Healpix
	using StaticArrays
	using Tables
end

# ╔═╡ 75169e36-9a85-42eb-a9d4-bfb4e9921e50
using PyCall

# ╔═╡ 0e7eacfe-febc-443c-b615-3063c9781f86
begin
	using PyPlot: plt
	using DirectionalStatistics: Circular
	using RectiGrids
end

# ╔═╡ 3b33fcb9-1e12-4318-b3e4-d00be59823d4
show_private = false

# ╔═╡ f4a906a6-4c65-42c6-8b03-665078490059
function is_notebook_local(filesrc)
	if isdefined(Main,:PlutoRunner)
		cell_id = tryparse(Base.UUID,last(filesrc,36))
		cell_id !== nothing && cell_id === Main.PlutoRunner.currently_running_cell_id[] && return true
	end
	return false
end

# ╔═╡ 02004717-405f-4fcc-bcfb-c36df5191259
macro pl(ex)
	is_notebook_local(String(__source__.file::Symbol)) ? esc(ex) : nothing
end

# ╔═╡ 7d0fecaf-3c17-42a1-9084-43164a032307
@pl begin
	using PyPlotUtils
	pyplot_style!()
end

# ╔═╡ a9b238cd-53c7-4099-b26b-cd8c2aca871a
@pl using AstroIceCube

# ╔═╡ e077e09b-b6eb-43ed-aad5-9b33d42435cd
@pl using AstroANTARES

# ╔═╡ b82dc5ed-8347-42cd-91bf-3fd463e5bf93
@pl begin
	import Pkg
	using PlutoUI: LocalResource
	# to check visually
	show_private && LocalResource(joinpath(Pkg.Artifacts.artifact_path(Pkg.Artifacts.artifact_hash("private_07_20_map", Pkg.Artifacts.find_artifacts_toml(pathof(AstroANTARES)))), "pvalue-map.png"))
end

# ╔═╡ d4ceeca4-0c6b-4bd5-bf03-6094dfa09bed


# ╔═╡ f015fb8e-f6de-43f5-b1c2-ffccd9996c2d


# ╔═╡ 234380c0-5b91-43d3-907f-1dbf9be959df
abstract type AbstractSkyMap end

# ╔═╡ c76662bd-fd1b-4042-ae6e-8337ec36ce5f
begin
	struct SkyMapHealpix{HMT} <: AbstractSkyMap
		hpix::HMT
	end
	
	ix_to_coords(m::SkyMapHealpix, ix::Int) = let
		codec, ra = Healpix.pix2ang(m.hpix, ix)
		return SVector(ra, Healpix.colat2lat(codec))
	end
	find_closest(m::SkyMapHealpix, (ra, dec)) = Healpix.ang2pix(m.hpix, Healpix.lat2colat(dec), ra)	
	(m::SkyMapHealpix)(radec) = m.hpix[find_closest(m, radec)]
	
	Tables.istable(m::SkyMapHealpix) = true	
	Tables.rowaccess(m::SkyMapHealpix) = true
	Tables.rows(m::SkyMapHealpix) = map(ix -> (coords=ix_to_coords(m, ix), value=m.hpix[ix]), 1:length(m.hpix))
end

# ╔═╡ f46bb1ef-d315-4213-b49a-48b9948d9443
begin
	skymap(hm::Healpix.HealpixMap) = SkyMapHealpix(hm)
	skymap_healpix(path::AbstractString, column=1, t=Float64) = skymap(Healpix.readMapFromFITS(path, column, t))
end

# ╔═╡ f0832587-526e-4683-9d4a-d2a6b734e06a


# ╔═╡ 58d41e3a-f5b9-4a38-970f-5f6ed7af85dd
@pl hm = skymap_healpix(dataset(IceCube.Map7yr()));

# ╔═╡ 610e3f20-ea38-4a0b-9611-49664f7589ff
@pl rowtable(hm)

# ╔═╡ e07f02ac-1da0-4425-9f94-279fa21725ed


# ╔═╡ 692d515c-2338-4382-acb7-149daad11cea
begin
	struct SkyMapROOT <: AbstractSkyMap
		pixedges::NTuple{2, Vector{Float64}}
		values::Matrix{Float64}
	end
	
	function skymap_root(path::AbstractString)
		uproot = pyimport_conda("uproot", "uproot", "conda-forge")
		@pywith uproot.open(path) as obj begin
			@assert !obj.weighted
			@assert unique(obj.axes[1].widths()) == [0.2]
			@assert unique(obj.axes[2].widths()) == [0.2]
	
			edges = map(obj.axes) do ax
				edges = ax.low:ax.width:ax.high
				@assert edges ≈ ax.edges()
				deg2rad.(edges)
			end
			values = obj.values()
			
			@assert length(edges[1]) == size(values, 1) + 1
			@assert length(edges[2]) == size(values, 2) + 1
	
			return SkyMapROOT(edges, values)
		end
	end
	
	ix_to_coords(m::SkyMapROOT, ix::CartesianIndex{2}) = let
		(SVector(m.pixedges[1][ix[1]], m.pixedges[2][ix[2]]) + SVector(m.pixedges[1][ix[1] + 1], m.pixedges[2][ix[2] + 1])) / 2
	end
	function find_closest(m::SkyMapROOT, (ra, dec))
		radec = ((ra + 2π) % 2π, dec)
		ixs = ntuple(2) do i
			ix = searchsortedlast(m.pixedges[i], radec[i])
			if ix == length(m.pixedges[i])
				length(m.pixedges[i]) - 1
			else
				es = m.pixedges[i][ix], m.pixedges[i][ix + 1]
				@assert es[1] <= radec[i] < es[2]
				ix
			end
		end |> CartesianIndex
	end
	(m::SkyMapROOT)(radec) = m.values[find_closest(m, radec)]
	
	Tables.istable(m::SkyMapROOT) = true	
	Tables.rowaccess(m::SkyMapROOT) = true
	Tables.rows(m::SkyMapROOT) = map(CartesianIndices(m.values)) do ix
		(
			coords=ix_to_coords(m, ix),
			value=m.values[ix])
	end
end

# ╔═╡ 42b0d114-eab1-4346-9fed-d2842c6646d1


# ╔═╡ 65a70ace-3ecf-44fe-bab2-376cc9b8a6f0
@pl if show_private
	rsm = skymap_root(dataset(ANTARES.MapPrivate()))
end

# ╔═╡ 2a258390-73b5-4601-9390-73ff081381d2
@pl rowtable(rsm)

# ╔═╡ acad3ea1-0edb-4492-a987-363679a7e995


# ╔═╡ 6ded72ca-5697-4001-a457-4abbdd25521e


# ╔═╡ 0f9d0df8-4a02-4309-86ef-07b3725ee0c0
function pyplot(sm::AbstractSkyMap; npix, do_ticks=true, kwargs...)
	ra_func = x -> Circular.center_angle(-x)

	M = let
		G = grid(ra=range(-π, π, length=2*npix), dec=range(-π/2, π/2, length=npix))
		map(sm, KeyedArray(G))
	end
	
	plt.pcolormesh(
		ra_func.(axiskeys(M, :ra)),
		axiskeys(M, :dec),
		permutedims(M, (:dec, :ra));
		kwargs...
	)
	
	if do_ticks
		for radeg in (-180:45:180)[2:end]
			ra_plt = ra_func(deg2rad(radeg))
			if ra_plt ≈ -π || ra_plt ≈ π
				plt.text(π, 0, "$(radeg)°")
				plt.text(-π, 0, "$(radeg)°")
			else
				plt.text(ra_plt, 0, "$(radeg)°")
			end
		end
		plt.xticks(deg2rad.(-180:22.5:180), [])
	end
	
	plt.grid(color=:grey)
	plt.gca().set_axisbelow(false)
	
	plt.gcf()
end

# ╔═╡ cdd5db56-3274-4332-bb99-e630d36d6dae
@pl let
    plt.figure(figsize=(15, 6))
    plt.subplot(111, projection="mollweide")

	pyplot(hm; npix=1000, cmap=:gnuplot2_r, vmax=4)
	plt.colorbar()
	
	plt.gcf()
end

# ╔═╡ 779c91bb-0c87-4ad5-810a-9ebbbf1a03a9
@pl let
    plt.figure(figsize=(15, 6))
    plt.subplot(111, projection="mollweide")
	
	pyplot(rsm; npix=500, cmap=:gnuplot2_r)
	plt.colorbar()
	
	plt.gcf()
end

# ╔═╡ 5e0023b6-b75e-4d08-aa30-df3e3107c364


# ╔═╡ 00000000-0000-0000-0000-000000000001
PLUTO_PROJECT_TOML_CONTENTS = """
[deps]
AstroANTARES = "0da805fc-ba30-469d-9680-7c125b5e0fb5"
AstroIceCube = "bdb22d2a-9ec9-4304-9ffb-84713910f94c"
DirectionalStatistics = "e814f24e-44b0-11e9-2fd5-aba2b6113d95"
Healpix = "9f4e344d-96bc-545a-84a3-ae6b9e1b672b"
Pkg = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"
PlutoUI = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
PyCall = "438e738f-606a-5dbb-bf0a-cddfbfd45ab0"
PyPlot = "d330b81b-6aea-500a-939a-2ce795aea3ee"
PyPlotUtils = "5384e752-6c47-47b3-86ac-9d091b110b31"
RectiGrids = "8ac6971d-971d-971d-971d-971d5ab1a71a"
StaticArrays = "90137ffa-7385-5640-81b9-e52037218182"
Tables = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"

[compat]
AstroANTARES = "~0.1.3"
AstroIceCube = "~0.1.2"
DirectionalStatistics = "~0.1.7"
Healpix = "~3.0.0"
PlutoUI = "~0.7.9"
PyCall = "~1.92.3"
PyPlot = "~2.9.0"
PyPlotUtils = "~0.1.1"
RectiGrids = "~0.1.2"
StaticArrays = "~1.2.8"
Tables = "~1.4.4"
"""

# ╔═╡ 00000000-0000-0000-0000-000000000002
PLUTO_MANIFEST_TOML_CONTENTS = """
# This file is machine-generated - editing it directly is not advised

[[AbstractFFTs]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "485ee0867925449198280d4af84bdb46a2a404d0"
uuid = "621f4979-c628-5d54-868e-fcf4e3e8185c"
version = "1.0.1"

[[Adapt]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "84918055d15b3114ede17ac6a7182f68870c16f7"
uuid = "79e6a3ab-5dfb-504d-930d-738a2a938a0e"
version = "3.3.1"

[[ArgTools]]
uuid = "0dad84c5-d112-42e6-8d28-ef12dabb789f"

[[ArrayInterface]]
deps = ["IfElse", "LinearAlgebra", "Requires", "SparseArrays", "Static"]
git-tree-sha1 = "a71d224f61475b93c9e196e83c17c6ac4dedacfa"
uuid = "4fba245c-0d91-5ea0-9b3e-6abc04ee57a9"
version = "3.1.18"

[[Artifacts]]
uuid = "56f22d72-fd6d-98f1-02f0-08ddc0907c33"

[[AstroANTARES]]
deps = ["AstroCatalogUtils", "AstroDataBase", "CSV", "DataPipes", "DateFormats", "InteractiveUtils", "LazyArtifacts", "Markdown", "Reexport", "StaticArrays"]
git-tree-sha1 = "5bb2b11bb7d87f54dc874cad6433cd0cf27c878a"
uuid = "0da805fc-ba30-469d-9680-7c125b5e0fb5"
version = "0.1.3"

[[AstroCatalogUtils]]
deps = ["AstroDataBase", "DirectionalStatistics", "Distances", "DocStringExtensions", "LinearAlgebra", "NearestNeighbors", "Parameters", "Setfield", "StaticArrays"]
git-tree-sha1 = "c069391ff862d8f3c396c27a3459b0a1b23edfd8"
uuid = "705829cc-62c6-4882-a4ec-bd7361a5ccd3"
version = "0.2.2"

[[AstroDataBase]]
deps = ["AxisKeys", "Dates", "OrderedCollections"]
git-tree-sha1 = "6a1e8aaa3eebb6077dbd61140bacee6721225bee"
uuid = "cc88de29-2d75-48be-97ee-dd03a6b97b76"
version = "0.1.19"

[[AstroIceCube]]
deps = ["AstroCatalogUtils", "AstroDataBase", "CodecZstd", "DataPipes", "DateFormats", "FixedWidthTables", "Glob", "InteractiveUtils", "LazyArtifacts", "Markdown", "Reexport", "StaticArrays"]
git-tree-sha1 = "e3cdc349194203ad5371b124f393ac83bce89aae"
uuid = "bdb22d2a-9ec9-4304-9ffb-84713910f94c"
version = "0.1.2"

[[AxisKeys]]
deps = ["AbstractFFTs", "CovarianceEstimation", "IntervalSets", "InvertedIndices", "LazyStack", "LinearAlgebra", "NamedDims", "OffsetArrays", "Statistics", "StatsBase", "Tables"]
git-tree-sha1 = "f7a17e0381cd8e627d9793433074823a2c1d7096"
uuid = "94b1ba4f-4ee9-5380-92f1-94cde586c3c5"
version = "0.1.18"

[[Base64]]
uuid = "2a0f44e3-6c83-55bd-87e4-b1978d98bd5f"

[[CFITSIO]]
deps = ["CFITSIO_jll"]
git-tree-sha1 = "7a54220c2a310e7c791e6992fe3a1a8a91154b1e"
uuid = "3b1b4be9-1499-4b22-8d78-7db3344d1961"
version = "1.1.1"

[[CFITSIO_jll]]
deps = ["Artifacts", "JLLWrappers", "LibCURL_jll", "Libdl", "Pkg"]
git-tree-sha1 = "2fabb5fc48d185d104ca7ed7444b475705993447"
uuid = "b3e40c51-02ae-5482-8a39-3ace5868dcf4"
version = "3.49.1+0"

[[CSV]]
deps = ["Dates", "Mmap", "Parsers", "PooledArrays", "SentinelArrays", "Tables", "Unicode"]
git-tree-sha1 = "b83aa3f513be680454437a0eee21001607e5d983"
uuid = "336ed68f-0bac-5ca0-87d4-7b16caf5d00b"
version = "0.8.5"

[[CodecZstd]]
deps = ["TranscodingStreams", "Zstd_jll"]
git-tree-sha1 = "d19cd9ae79ef31774151637492291d75194fc5fa"
uuid = "6b39b394-51ab-5f42-8807-6242bab2b4c2"
version = "0.7.0"

[[ColorTypes]]
deps = ["FixedPointNumbers", "Random"]
git-tree-sha1 = "024fe24d83e4a5bf5fc80501a314ce0d1aa35597"
uuid = "3da002f7-5984-5a60-b8a6-cbb66c0b333f"
version = "0.11.0"

[[Colors]]
deps = ["ColorTypes", "FixedPointNumbers", "Reexport"]
git-tree-sha1 = "417b0ed7b8b838aa6ca0a87aadf1bb9eb111ce40"
uuid = "5ae59095-9a9b-59fe-a467-6f913c188581"
version = "0.12.8"

[[Compat]]
deps = ["Base64", "Dates", "DelimitedFiles", "Distributed", "InteractiveUtils", "LibGit2", "Libdl", "LinearAlgebra", "Markdown", "Mmap", "Pkg", "Printf", "REPL", "Random", "SHA", "Serialization", "SharedArrays", "Sockets", "SparseArrays", "Statistics", "Test", "UUIDs", "Unicode"]
git-tree-sha1 = "dc7dedc2c2aa9faf59a55c622760a25cbefbe941"
uuid = "34da2185-b29b-5c13-b0c7-acf172513d20"
version = "3.31.0"

[[CompilerSupportLibraries_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "e66e0078-7015-5450-92f7-15fbd957f2ae"

[[Conda]]
deps = ["JSON", "VersionParsing"]
git-tree-sha1 = "299304989a5e6473d985212c28928899c74e9421"
uuid = "8f4d0f93-b110-5947-807f-2305c1781a2d"
version = "1.5.2"

[[ConstructionBase]]
deps = ["LinearAlgebra"]
git-tree-sha1 = "f74e9d5388b8620b4cee35d4c5a618dd4dc547f4"
uuid = "187b0558-2788-49d3-abe0-74a17ed4e7c9"
version = "1.3.0"

[[CovarianceEstimation]]
deps = ["LinearAlgebra", "Statistics", "StatsBase"]
git-tree-sha1 = "bc3930158d2be029e90b7c40d1371c4f54fa04db"
uuid = "587fd27a-f159-11e8-2dae-1979310e6154"
version = "0.2.6"

[[DataAPI]]
git-tree-sha1 = "ee400abb2298bd13bfc3df1c412ed228061a2385"
uuid = "9a962f9c-6df0-11e9-0e5d-c546b8b5ee8a"
version = "1.7.0"

[[DataPipes]]
deps = ["SplitApplyCombine"]
git-tree-sha1 = "6f88db1e9fb926ccb663ef86abe21d9687322656"
uuid = "02685ad9-2d12-40c3-9f73-c6aeda6a7ff5"
version = "0.1.9"

[[DataStructures]]
deps = ["Compat", "InteractiveUtils", "OrderedCollections"]
git-tree-sha1 = "4437b64df1e0adccc3e5d1adbc3ac741095e4677"
uuid = "864edb3b-99cc-5e75-8d2d-829cb0a9cfe8"
version = "0.18.9"

[[DataValueInterfaces]]
git-tree-sha1 = "bfc1187b79289637fa0ef6d4436ebdfe6905cbd6"
uuid = "e2d170a0-9d28-54be-80f0-106bbe20a464"
version = "1.0.0"

[[DateFormats]]
deps = ["Dates", "DocStringExtensions"]
git-tree-sha1 = "9814d405f6a3b986a67ac636952016ab1db62f44"
uuid = "44557152-fe0a-4de1-8405-416d90313ce6"
version = "0.1.12"

[[Dates]]
deps = ["Printf"]
uuid = "ade2ca70-3891-5945-98fb-dc099432e06a"

[[DelimitedFiles]]
deps = ["Mmap"]
uuid = "8bb1440f-4735-579b-a4ab-409b98df4dab"

[[Dictionaries]]
deps = ["Indexing", "Random"]
git-tree-sha1 = "011d10589449681ad631c0f866a2ce72771648c6"
uuid = "85a47980-9c8c-11e8-2b9f-f7ca1fa99fb4"
version = "0.3.10"

[[DirectionalStatistics]]
deps = ["IntervalSets", "LinearAlgebra", "Statistics", "StatsBase"]
git-tree-sha1 = "9a36d3ca402caba33bae2dca156662c3c0fe3f25"
uuid = "e814f24e-44b0-11e9-2fd5-aba2b6113d95"
version = "0.1.7"

[[Distances]]
deps = ["LinearAlgebra", "Statistics", "StatsAPI"]
git-tree-sha1 = "abe4ad222b26af3337262b8afb28fab8d215e9f8"
uuid = "b4f34e82-e78d-54a5-968a-f98e89d6e8f7"
version = "0.10.3"

[[Distributed]]
deps = ["Random", "Serialization", "Sockets"]
uuid = "8ba89e20-285c-5b6f-9357-94700520ee1b"

[[DocStringExtensions]]
deps = ["LibGit2"]
git-tree-sha1 = "a32185f5428d3986f47c2ab78b1f216d5e6cc96f"
uuid = "ffbed154-4ef7-542d-bbb7-c09d3a79fcae"
version = "0.8.5"

[[Downloads]]
deps = ["ArgTools", "LibCURL", "NetworkOptions"]
uuid = "f43a241f-c20a-4ad4-852c-f6b1247861c6"

[[EllipsisNotation]]
deps = ["ArrayInterface"]
git-tree-sha1 = "8041575f021cba5a099a456b4163c9a08b566a02"
uuid = "da5c29d0-fa7d-589e-88eb-ea29b0a81949"
version = "1.1.0"

[[FixedPointNumbers]]
deps = ["Statistics"]
git-tree-sha1 = "335bfdceacc84c5cdf16aadc768aa5ddfc5383cc"
uuid = "53c48c17-4a7d-5ca2-90c5-79b7896eea93"
version = "0.8.4"

[[FixedWidthTables]]
deps = ["IterTools"]
git-tree-sha1 = "278be7e1e6185010d658ee4ff607a23c76ab7a14"
uuid = "ef6c9296-b25d-482c-a0a7-898918678e13"
version = "0.1.4"

[[Future]]
deps = ["Random"]
uuid = "9fa8497b-333b-5362-9e8d-4d0656e87820"

[[Glob]]
git-tree-sha1 = "4df9f7e06108728ebf00a0a11edee4b29a482bb2"
uuid = "c27321d9-0574-5035-807b-f59d2c89b15c"
version = "1.3.0"

[[Healpix]]
deps = ["CFITSIO", "LazyArtifacts", "Libsharp", "Pkg", "Printf", "RecipesBase", "Test"]
git-tree-sha1 = "d258dae11794a5fb5b64ed4b3968a88450e5841b"
uuid = "9f4e344d-96bc-545a-84a3-ae6b9e1b672b"
version = "3.0.0"

[[IfElse]]
git-tree-sha1 = "28e837ff3e7a6c3cdb252ce49fb412c8eb3caeef"
uuid = "615f187c-cbe4-4ef1-ba3b-2fcf58d6d173"
version = "0.1.0"

[[Indexing]]
git-tree-sha1 = "ce1566720fd6b19ff3411404d4b977acd4814f9f"
uuid = "313cdc1a-70c2-5d6a-ae34-0150d3930a38"
version = "1.1.1"

[[InteractiveUtils]]
deps = ["Markdown"]
uuid = "b77e0a4c-d291-57a0-90e8-8db25a27a240"

[[IntervalSets]]
deps = ["Dates", "EllipsisNotation", "Statistics"]
git-tree-sha1 = "3cc368af3f110a767ac786560045dceddfc16758"
uuid = "8197267c-284f-5f27-9208-e0e47529a953"
version = "0.5.3"

[[InvertedIndices]]
deps = ["Test"]
git-tree-sha1 = "15732c475062348b0165684ffe28e85ea8396afc"
uuid = "41ab1584-1d38-5bbf-9106-f11c6c58b48f"
version = "1.0.0"

[[IterTools]]
git-tree-sha1 = "05110a2ab1fc5f932622ffea2a003221f4782c18"
uuid = "c8e1da08-722c-5040-9ed9-7db0dc04731e"
version = "1.3.0"

[[IteratorInterfaceExtensions]]
git-tree-sha1 = "a3f24677c21f5bbe9d2a714f95dcd58337fb2856"
uuid = "82899510-4779-5014-852e-03e436cf321d"
version = "1.0.0"

[[JLLWrappers]]
deps = ["Preferences"]
git-tree-sha1 = "642a199af8b68253517b80bd3bfd17eb4e84df6e"
uuid = "692b3bcd-3c85-4b1f-b108-f13ce0eb3210"
version = "1.3.0"

[[JSON]]
deps = ["Dates", "Mmap", "Parsers", "Unicode"]
git-tree-sha1 = "81690084b6198a2e1da36fcfda16eeca9f9f24e4"
uuid = "682c06a0-de6a-54ab-a142-c8b1cf79cde6"
version = "0.21.1"

[[LaTeXStrings]]
git-tree-sha1 = "c7f1c695e06c01b95a67f0cd1d34994f3e7db104"
uuid = "b964fa9f-0449-5b57-a5c2-d3ea65f4040f"
version = "1.2.1"

[[LazyArtifacts]]
deps = ["Artifacts", "Pkg"]
uuid = "4af54fe1-eca0-43a8-85a7-787d91b784e3"

[[LazyStack]]
deps = ["LinearAlgebra", "NamedDims", "OffsetArrays", "Test", "ZygoteRules"]
git-tree-sha1 = "a8bf67afad3f1ee59d367267adb7c44ccac7fdee"
uuid = "1fad7336-0346-5a1a-a56f-a06ba010965b"
version = "0.0.7"

[[LibCURL]]
deps = ["LibCURL_jll", "MozillaCACerts_jll"]
uuid = "b27032c2-a3e7-50c8-80cd-2d36dbcbfd21"

[[LibCURL_jll]]
deps = ["Artifacts", "LibSSH2_jll", "Libdl", "MbedTLS_jll", "Zlib_jll", "nghttp2_jll"]
uuid = "deac9b47-8bc7-5906-a0fe-35ac56dc84c0"

[[LibGit2]]
deps = ["Base64", "NetworkOptions", "Printf", "SHA"]
uuid = "76f85450-5226-5b5a-8eaa-529ad045b433"

[[LibSSH2_jll]]
deps = ["Artifacts", "Libdl", "MbedTLS_jll"]
uuid = "29816b5a-b9ab-546f-933c-edad1886dfa8"

[[Libdl]]
uuid = "8f399da3-3557-5675-b5ff-fb832c97cbdb"

[[Libsharp]]
deps = ["Libdl", "libsharp2_jll"]
git-tree-sha1 = "02f955af0f2fcc172fc7c37c3a52360797de8043"
uuid = "ac8d63fe-4615-43ae-9860-9cd4a3820532"
version = "0.1.0"

[[LinearAlgebra]]
deps = ["Libdl"]
uuid = "37e2e46d-f89d-539d-b4ee-838fcccc9c8e"

[[Logging]]
uuid = "56ddb016-857b-54e1-b83d-db4d58db5568"

[[MacroTools]]
deps = ["Markdown", "Random"]
git-tree-sha1 = "6a8a2a625ab0dea913aba95c11370589e0239ff0"
uuid = "1914dd2f-81c6-5fcd-8719-6d5c9610ff09"
version = "0.5.6"

[[Markdown]]
deps = ["Base64"]
uuid = "d6f4376e-aef5-505a-96c1-9c027394607a"

[[MbedTLS_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "c8ffd9c3-330d-5841-b78e-0817d7145fa1"

[[Missings]]
deps = ["DataAPI"]
git-tree-sha1 = "4ea90bd5d3985ae1f9a908bd4500ae88921c5ce7"
uuid = "e1d29d7a-bbdc-5cf2-9ac0-f12de2c33e28"
version = "1.0.0"

[[Mmap]]
uuid = "a63ad114-7e13-5084-954f-fe012c677804"

[[MozillaCACerts_jll]]
uuid = "14a3606d-f60d-562e-9121-12d972cd8159"

[[NamedDims]]
deps = ["AbstractFFTs", "LinearAlgebra", "Pkg", "Requires", "Statistics"]
git-tree-sha1 = "52985b34519b12fd0dcebbe34e74b2dbe6d03183"
uuid = "356022a1-0364-5f58-8944-0da4b18d706f"
version = "0.2.35"

[[NearestNeighbors]]
deps = ["Distances", "StaticArrays"]
git-tree-sha1 = "16baacfdc8758bc374882566c9187e785e85c2f0"
uuid = "b8a86587-4115-5ab1-83bc-aa920d37bbce"
version = "0.4.9"

[[NetworkOptions]]
uuid = "ca575930-c2e3-43a9-ace4-1e988b2c1908"

[[OffsetArrays]]
deps = ["Adapt"]
git-tree-sha1 = "4f825c6da64aebaa22cc058ecfceed1ab9af1c7e"
uuid = "6fe1bfb0-de20-5000-8ca7-80f57d26f881"
version = "1.10.3"

[[OrderedCollections]]
git-tree-sha1 = "85f8e6578bf1f9ee0d11e7bb1b1456435479d47c"
uuid = "bac558e1-5e72-5ebc-8fee-abe8a469f55d"
version = "1.4.1"

[[Parameters]]
deps = ["OrderedCollections", "UnPack"]
git-tree-sha1 = "2276ac65f1e236e0a6ea70baff3f62ad4c625345"
uuid = "d96e819e-fc66-5662-9728-84c9c7592b0a"
version = "0.12.2"

[[Parsers]]
deps = ["Dates"]
git-tree-sha1 = "c8abc88faa3f7a3950832ac5d6e690881590d6dc"
uuid = "69de0a69-1ddd-5017-9359-2bf0b02dc9f0"
version = "1.1.0"

[[Pkg]]
deps = ["Artifacts", "Dates", "Downloads", "LibGit2", "Libdl", "Logging", "Markdown", "Printf", "REPL", "Random", "SHA", "Serialization", "TOML", "Tar", "UUIDs", "p7zip_jll"]
uuid = "44cfe95a-1eb2-52ea-b672-e2afdf69b78f"

[[PlutoUI]]
deps = ["Base64", "Dates", "InteractiveUtils", "JSON", "Logging", "Markdown", "Random", "Reexport", "Suppressor"]
git-tree-sha1 = "44e225d5837e2a2345e69a1d1e01ac2443ff9fcb"
uuid = "7f904dfe-b85e-4ff6-b463-dae2292396a8"
version = "0.7.9"

[[PooledArrays]]
deps = ["DataAPI", "Future"]
git-tree-sha1 = "cde4ce9d6f33219465b55162811d8de8139c0414"
uuid = "2dfb63ee-cc39-5dd5-95bd-886bf059d720"
version = "1.2.1"

[[Preferences]]
deps = ["TOML"]
git-tree-sha1 = "00cfd92944ca9c760982747e9a1d0d5d86ab1e5a"
uuid = "21216c6a-2e73-6563-6e65-726566657250"
version = "1.2.2"

[[Printf]]
deps = ["Unicode"]
uuid = "de0858da-6303-5e67-8744-51eddeeeb8d7"

[[PyCall]]
deps = ["Conda", "Dates", "Libdl", "LinearAlgebra", "MacroTools", "Serialization", "VersionParsing"]
git-tree-sha1 = "169bb8ea6b1b143c5cf57df6d34d022a7b60c6db"
uuid = "438e738f-606a-5dbb-bf0a-cddfbfd45ab0"
version = "1.92.3"

[[PyPlot]]
deps = ["Colors", "LaTeXStrings", "PyCall", "Sockets", "Test", "VersionParsing"]
git-tree-sha1 = "67dde2482fe1a72ef62ed93f8c239f947638e5a2"
uuid = "d330b81b-6aea-500a-939a-2ce795aea3ee"
version = "2.9.0"

[[PyPlotUtils]]
deps = ["AxisKeys", "OffsetArrays", "PyCall", "PyPlot", "StatsBase", "Unitful"]
git-tree-sha1 = "8a150b005c47df79ab74e234be699345296ec88c"
uuid = "5384e752-6c47-47b3-86ac-9d091b110b31"
version = "0.1.1"

[[REPL]]
deps = ["InteractiveUtils", "Markdown", "Sockets", "Unicode"]
uuid = "3fa0cd96-eef1-5676-8a61-b3b8758bbffb"

[[Random]]
deps = ["Serialization"]
uuid = "9a3f8284-a2c9-5f02-9a11-845980a1fd5c"

[[RecipesBase]]
git-tree-sha1 = "b3fb709f3c97bfc6e948be68beeecb55a0b340ae"
uuid = "3cdcf5f2-1ef4-517c-9805-6587b60abb01"
version = "1.1.1"

[[RectiGrids]]
deps = ["AxisKeys", "Random"]
git-tree-sha1 = "19d2340eb628233e15c6c0e697f70a49b40d7706"
uuid = "8ac6971d-971d-971d-971d-971d5ab1a71a"
version = "0.1.2"

[[Reexport]]
git-tree-sha1 = "5f6c21241f0f655da3952fd60aa18477cf96c220"
uuid = "189a3867-3050-52da-a836-e630ba90ab69"
version = "1.1.0"

[[Requires]]
deps = ["UUIDs"]
git-tree-sha1 = "4036a3bd08ac7e968e27c203d45f5fff15020621"
uuid = "ae029012-a4dd-5104-9daa-d747884805df"
version = "1.1.3"

[[SHA]]
uuid = "ea8e919c-243c-51af-8825-aaa63cd721ce"

[[SentinelArrays]]
deps = ["Dates", "Random"]
git-tree-sha1 = "35927c2c11da0a86bcd482464b93dadd09ce420f"
uuid = "91c51154-3ec4-41a3-a24f-3f23e20d615c"
version = "1.3.5"

[[Serialization]]
uuid = "9e88b42a-f829-5b0c-bbe9-9e923198166b"

[[Setfield]]
deps = ["ConstructionBase", "Future", "MacroTools", "Requires"]
git-tree-sha1 = "d5640fc570fb1b6c54512f0bd3853866bd298b3e"
uuid = "efcf1570-3423-57d1-acb7-fd33fddbac46"
version = "0.7.0"

[[SharedArrays]]
deps = ["Distributed", "Mmap", "Random", "Serialization"]
uuid = "1a1011a3-84de-559e-8e89-a11a2f7dc383"

[[Sockets]]
uuid = "6462fe0b-24de-5631-8697-dd941f90decc"

[[SortingAlgorithms]]
deps = ["DataStructures"]
git-tree-sha1 = "b3363d7460f7d098ca0912c69b082f75625d7508"
uuid = "a2af1166-a08f-5f64-846c-94a0d3cef48c"
version = "1.0.1"

[[SparseArrays]]
deps = ["LinearAlgebra", "Random"]
uuid = "2f01184e-e22b-5df5-ae63-d93ebab69eaf"

[[SplitApplyCombine]]
deps = ["Dictionaries", "Indexing"]
git-tree-sha1 = "3cdd86a92737fa0c8af19aecb1141e71057dc2db"
uuid = "03a91e81-4c3e-53e1-a0a4-9c0c8f19dd66"
version = "1.1.4"

[[Static]]
deps = ["IfElse"]
git-tree-sha1 = "62701892d172a2fa41a1f829f66d2b0db94a9a63"
uuid = "aedffcd0-7271-4cad-89d0-dc628f76c6d3"
version = "0.3.0"

[[StaticArrays]]
deps = ["LinearAlgebra", "Random", "Statistics"]
git-tree-sha1 = "5b2f81eeb66bcfe379947c500aae773c85c31033"
uuid = "90137ffa-7385-5640-81b9-e52037218182"
version = "1.2.8"

[[Statistics]]
deps = ["LinearAlgebra", "SparseArrays"]
uuid = "10745b16-79ce-11e8-11f9-7d13ad32a3b2"

[[StatsAPI]]
git-tree-sha1 = "1958272568dc176a1d881acb797beb909c785510"
uuid = "82ae8749-77ed-4fe6-ae5f-f523153014b0"
version = "1.0.0"

[[StatsBase]]
deps = ["DataAPI", "DataStructures", "LinearAlgebra", "Missings", "Printf", "Random", "SortingAlgorithms", "SparseArrays", "Statistics", "StatsAPI"]
git-tree-sha1 = "2f6792d523d7448bbe2fec99eca9218f06cc746d"
uuid = "2913bbd2-ae8a-5f71-8c99-4fb6c76f3a91"
version = "0.33.8"

[[Suppressor]]
git-tree-sha1 = "a819d77f31f83e5792a76081eee1ea6342ab8787"
uuid = "fd094767-a336-5f1f-9728-57cf17d0bbfb"
version = "0.2.0"

[[TOML]]
deps = ["Dates"]
uuid = "fa267f1f-6049-4f14-aa54-33bafae1ed76"

[[TableTraits]]
deps = ["IteratorInterfaceExtensions"]
git-tree-sha1 = "c06b2f539df1c6efa794486abfb6ed2022561a39"
uuid = "3783bdb8-4a98-5b6b-af9a-565f29a5fe9c"
version = "1.0.1"

[[Tables]]
deps = ["DataAPI", "DataValueInterfaces", "IteratorInterfaceExtensions", "LinearAlgebra", "TableTraits", "Test"]
git-tree-sha1 = "8ed4a3ea724dac32670b062be3ef1c1de6773ae8"
uuid = "bd369af6-aec1-5ad0-b16a-f7cc5008161c"
version = "1.4.4"

[[Tar]]
deps = ["ArgTools", "SHA"]
uuid = "a4e569a6-e804-4fa4-b0f3-eef7a1d5b13e"

[[Test]]
deps = ["InteractiveUtils", "Logging", "Random", "Serialization"]
uuid = "8dfed614-e22c-5e08-85e1-65c5234f0b40"

[[TranscodingStreams]]
deps = ["Random", "Test"]
git-tree-sha1 = "7c53c35547de1c5b9d46a4797cf6d8253807108c"
uuid = "3bb67fe8-82b1-5028-8e26-92a6c54297fa"
version = "0.9.5"

[[UUIDs]]
deps = ["Random", "SHA"]
uuid = "cf7118a7-6976-5b1a-9a39-7adc72f591a4"

[[UnPack]]
git-tree-sha1 = "387c1f73762231e86e0c9c5443ce3b4a0a9a0c2b"
uuid = "3a884ed6-31ef-47d7-9d2a-63182c4928ed"
version = "1.0.2"

[[Unicode]]
uuid = "4ec0a83e-493e-50e2-b9ac-8f72acf5a8f5"

[[Unitful]]
deps = ["ConstructionBase", "Dates", "LinearAlgebra", "Random"]
git-tree-sha1 = "a981a8ef8714cba2fd9780b22fd7a469e7aaf56d"
uuid = "1986cc42-f94f-5a68-af5c-568840ba703d"
version = "1.9.0"

[[VersionParsing]]
git-tree-sha1 = "80229be1f670524750d905f8fc8148e5a8c4537f"
uuid = "81def892-9a0e-5fdd-b105-ffc91e053289"
version = "1.2.0"

[[Zlib_jll]]
deps = ["Libdl"]
uuid = "83775a58-1f1d-513f-b197-d71354ab007a"

[[Zstd_jll]]
deps = ["Artifacts", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "cc4bf3fdde8b7e3e9fa0351bdeedba1cf3b7f6e6"
uuid = "3161d3a3-bdf6-5164-811a-617609db77b4"
version = "1.5.0+0"

[[ZygoteRules]]
deps = ["MacroTools"]
git-tree-sha1 = "9e7a1e8ca60b742e508a315c17eef5211e7fbfd7"
uuid = "700de1a5-db45-46bc-99cf-38207098b444"
version = "0.2.1"

[[libsharp2_jll]]
deps = ["Artifacts", "CompilerSupportLibraries_jll", "JLLWrappers", "Libdl", "Pkg"]
git-tree-sha1 = "82d6e24d2acef1458532b4b7725bae138a91779d"
uuid = "180207a7-b08e-5162-af94-7d62a04fe081"
version = "1.0.1+0"

[[nghttp2_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "8e850ede-7688-5339-a07c-302acd2aaf8d"

[[p7zip_jll]]
deps = ["Artifacts", "Libdl"]
uuid = "3f19e933-33d8-53b3-aaab-bd5110c3b7a0"
"""

# ╔═╡ Cell order:
# ╠═3b33fcb9-1e12-4318-b3e4-d00be59823d4
# ╟─f4a906a6-4c65-42c6-8b03-665078490059
# ╟─02004717-405f-4fcc-bcfb-c36df5191259
# ╟─7d0fecaf-3c17-42a1-9084-43164a032307
# ╠═d4ceeca4-0c6b-4bd5-bf03-6094dfa09bed
# ╠═f2385a7e-64f1-47d4-bf9c-59855be81eca
# ╠═f015fb8e-f6de-43f5-b1c2-ffccd9996c2d
# ╠═234380c0-5b91-43d3-907f-1dbf9be959df
# ╠═c76662bd-fd1b-4042-ae6e-8337ec36ce5f
# ╠═f46bb1ef-d315-4213-b49a-48b9948d9443
# ╠═f0832587-526e-4683-9d4a-d2a6b734e06a
# ╠═a9b238cd-53c7-4099-b26b-cd8c2aca871a
# ╠═58d41e3a-f5b9-4a38-970f-5f6ed7af85dd
# ╠═610e3f20-ea38-4a0b-9611-49664f7589ff
# ╠═cdd5db56-3274-4332-bb99-e630d36d6dae
# ╠═e07f02ac-1da0-4425-9f94-279fa21725ed
# ╠═75169e36-9a85-42eb-a9d4-bfb4e9921e50
# ╠═692d515c-2338-4382-acb7-149daad11cea
# ╠═42b0d114-eab1-4346-9fed-d2842c6646d1
# ╠═e077e09b-b6eb-43ed-aad5-9b33d42435cd
# ╠═65a70ace-3ecf-44fe-bab2-376cc9b8a6f0
# ╠═2a258390-73b5-4601-9390-73ff081381d2
# ╠═779c91bb-0c87-4ad5-810a-9ebbbf1a03a9
# ╟─b82dc5ed-8347-42cd-91bf-3fd463e5bf93
# ╠═acad3ea1-0edb-4492-a987-363679a7e995
# ╠═6ded72ca-5697-4001-a457-4abbdd25521e
# ╠═0e7eacfe-febc-443c-b615-3063c9781f86
# ╠═0f9d0df8-4a02-4309-86ef-07b3725ee0c0
# ╠═5e0023b6-b75e-4d08-aa30-df3e3107c364
# ╟─00000000-0000-0000-0000-000000000001
# ╟─00000000-0000-0000-0000-000000000002
